// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.OperationalInsights;
using Microsoft.Azure.Management.OperationalInsights.Models;

namespace Microsoft.Azure.Management.OperationalInsights
{
    /// <summary>
    /// .Net client wrapper for the REST API for Azure Operational Insights
    /// </summary>
    public static partial class StorageInsightOperationsExtensions
    {
        /// <summary>
        /// Create or update a storage insight.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the storage insight.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the parent workspace that will contain the
        /// storage insight
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to create or update a storage
        /// insight.
        /// </param>
        /// <returns>
        /// The create or update storage insight operation response.
        /// </returns>
        public static StorageInsightCreateOrUpdateResponse CreateOrUpdate(this IStorageInsightOperations operations, string resourceGroupName, string workspaceName, StorageInsightCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IStorageInsightOperations)s).CreateOrUpdateAsync(resourceGroupName, workspaceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create or update a storage insight.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the storage insight.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the parent workspace that will contain the
        /// storage insight
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to create or update a storage
        /// insight.
        /// </param>
        /// <returns>
        /// The create or update storage insight operation response.
        /// </returns>
        public static Task<StorageInsightCreateOrUpdateResponse> CreateOrUpdateAsync(this IStorageInsightOperations operations, string resourceGroupName, string workspaceName, StorageInsightCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, workspaceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a storage insight instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the storage insight.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the storage
        /// insight.
        /// </param>
        /// <param name='storageInsightName'>
        /// Required. The name of the storage insight.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IStorageInsightOperations operations, string resourceGroupName, string workspaceName, string storageInsightName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IStorageInsightOperations)s).DeleteAsync(resourceGroupName, workspaceName, storageInsightName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes a storage insight instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the storage insight.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the storage
        /// insight.
        /// </param>
        /// <param name='storageInsightName'>
        /// Required. The name of the storage insight.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IStorageInsightOperations operations, string resourceGroupName, string workspaceName, string storageInsightName)
        {
            return operations.DeleteAsync(resourceGroupName, workspaceName, storageInsightName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets a storage insight instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the storage insight.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the storage
        /// insight.
        /// </param>
        /// <param name='storageInsightName'>
        /// Required. The name of the storage insight
        /// </param>
        /// <returns>
        /// The get storage insight operation response.
        /// </returns>
        public static StorageInsightGetResponse Get(this IStorageInsightOperations operations, string resourceGroupName, string workspaceName, string storageInsightName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IStorageInsightOperations)s).GetAsync(resourceGroupName, workspaceName, storageInsightName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets a storage insight instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the storage insight.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the storage
        /// insight.
        /// </param>
        /// <param name='storageInsightName'>
        /// Required. The name of the storage insight
        /// </param>
        /// <returns>
        /// The get storage insight operation response.
        /// </returns>
        public static Task<StorageInsightGetResponse> GetAsync(this IStorageInsightOperations operations, string resourceGroupName, string workspaceName, string storageInsightName)
        {
            return operations.GetAsync(resourceGroupName, workspaceName, storageInsightName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the first page of storage insight instances in a workspace
        /// with the link to the next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the storage insights.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The workspace that contains the storage insights.
        /// </param>
        /// <returns>
        /// The list storage insights operation response.
        /// </returns>
        public static StorageInsightListResponse ListInWorkspace(this IStorageInsightOperations operations, string resourceGroupName, string workspaceName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IStorageInsightOperations)s).ListInWorkspaceAsync(resourceGroupName, workspaceName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the first page of storage insight instances in a workspace
        /// with the link to the next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the storage insights.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The workspace that contains the storage insights.
        /// </param>
        /// <returns>
        /// The list storage insights operation response.
        /// </returns>
        public static Task<StorageInsightListResponse> ListInWorkspaceAsync(this IStorageInsightOperations operations, string resourceGroupName, string workspaceName)
        {
            return operations.ListInWorkspaceAsync(resourceGroupName, workspaceName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the next page of storage insight instances with the link to
        /// the next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The url to the next storage insights page.
        /// </param>
        /// <returns>
        /// The list storage insights operation response.
        /// </returns>
        public static StorageInsightListResponse ListNext(this IStorageInsightOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IStorageInsightOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the next page of storage insight instances with the link to
        /// the next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IStorageInsightOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The url to the next storage insights page.
        /// </param>
        /// <returns>
        /// The list storage insights operation response.
        /// </returns>
        public static Task<StorageInsightListResponse> ListNextAsync(this IStorageInsightOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
    }
}
