// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.WindowsAzure.Management.Network;
using Microsoft.WindowsAzure.Management.Network.Models;

namespace Microsoft.WindowsAzure.Management.Network
{
    /// <summary>
    /// The Service Management API includes operations for managing the virtual
    /// networks for your subscription.  (see
    /// http://msdn.microsoft.com/en-us/library/windowsazure/jj157182.aspx for
    /// more information)
    /// </summary>
    public static partial class ReservedIPOperationsExtensions
    {
        /// <summary>
        /// The Associate Reserved IP operation associates a Reserved IP with a
        /// service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='reservedIpName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to associate Reserved IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static OperationStatusResponse Associate(this IReservedIPOperations operations, string reservedIpName, NetworkReservedIPMobilityParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).AssociateAsync(reservedIpName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Associate Reserved IP operation associates a Reserved IP with a
        /// service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='reservedIpName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to associate Reserved IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<OperationStatusResponse> AssociateAsync(this IReservedIPOperations operations, string reservedIpName, NetworkReservedIPMobilityParameters parameters)
        {
            return operations.AssociateAsync(reservedIpName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The BeginAssociate begins to associate a Reserved IP with a service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='reservedIpName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin associating Reserved IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static OperationStatusResponse BeginAssociating(this IReservedIPOperations operations, string reservedIpName, NetworkReservedIPMobilityParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).BeginAssociatingAsync(reservedIpName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The BeginAssociate begins to associate a Reserved IP with a service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='reservedIpName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin associating Reserved IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<OperationStatusResponse> BeginAssociatingAsync(this IReservedIPOperations operations, string reservedIpName, NetworkReservedIPMobilityParameters parameters)
        {
            return operations.BeginAssociatingAsync(reservedIpName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Begin Creating Reserved IP operation creates a reserved IP from
        /// your the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Creating Reserved IP
        /// operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static OperationStatusResponse BeginCreating(this IReservedIPOperations operations, NetworkReservedIPCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).BeginCreatingAsync(parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Begin Creating Reserved IP operation creates a reserved IP from
        /// your the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Creating Reserved IP
        /// operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<OperationStatusResponse> BeginCreatingAsync(this IReservedIPOperations operations, NetworkReservedIPCreateParameters parameters)
        {
            return operations.BeginCreatingAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Begin Deleting Reserved IP operation removes a reserved IP from
        /// your the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='ipName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse BeginDeleting(this IReservedIPOperations operations, string ipName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).BeginDeletingAsync(ipName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Begin Deleting Reserved IP operation removes a reserved IP from
        /// your the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='ipName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> BeginDeletingAsync(this IReservedIPOperations operations, string ipName)
        {
            return operations.BeginDeletingAsync(ipName, CancellationToken.None);
        }
        
        /// <summary>
        /// The BeginDisassociate begins to disassociate a Reserved IP from a
        /// service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='reservedIpName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin disassociating Reserved
        /// IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static OperationStatusResponse BeginDisassociating(this IReservedIPOperations operations, string reservedIpName, NetworkReservedIPMobilityParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).BeginDisassociatingAsync(reservedIpName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The BeginDisassociate begins to disassociate a Reserved IP from a
        /// service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='reservedIpName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin disassociating Reserved
        /// IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<OperationStatusResponse> BeginDisassociatingAsync(this IReservedIPOperations operations, string reservedIpName, NetworkReservedIPMobilityParameters parameters)
        {
            return operations.BeginDisassociatingAsync(reservedIpName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Create Reserved IP operation creates a reserved IP from your
        /// the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Reserved IP operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static OperationStatusResponse Create(this IReservedIPOperations operations, NetworkReservedIPCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).CreateAsync(parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Create Reserved IP operation creates a reserved IP from your
        /// the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Reserved IP operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<OperationStatusResponse> CreateAsync(this IReservedIPOperations operations, NetworkReservedIPCreateParameters parameters)
        {
            return operations.CreateAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Delete Reserved IP operation removes a reserved IP from your
        /// the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='ipName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static OperationStatusResponse Delete(this IReservedIPOperations operations, string ipName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).DeleteAsync(ipName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Delete Reserved IP operation removes a reserved IP from your
        /// the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='ipName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<OperationStatusResponse> DeleteAsync(this IReservedIPOperations operations, string ipName)
        {
            return operations.DeleteAsync(ipName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Disassociate Reserved IP operation disassociates a Reserved IP
        /// from a service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='reservedIpName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to disassociate Reserved IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static OperationStatusResponse Disassociate(this IReservedIPOperations operations, string reservedIpName, NetworkReservedIPMobilityParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).DisassociateAsync(reservedIpName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Disassociate Reserved IP operation disassociates a Reserved IP
        /// from a service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='reservedIpName'>
        /// Required. The name of the reserved IP.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to disassociate Reserved IP.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<OperationStatusResponse> DisassociateAsync(this IReservedIPOperations operations, string reservedIpName, NetworkReservedIPMobilityParameters parameters)
        {
            return operations.DisassociateAsync(reservedIpName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get Reserved IP operation retrieves the details for the virtual
        /// IP reserved for the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='ipName'>
        /// Required. The name of the reserved IP to retrieve.
        /// </param>
        /// <returns>
        /// A reserved IP associated with your subscription.
        /// </returns>
        public static NetworkReservedIPGetResponse Get(this IReservedIPOperations operations, string ipName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).GetAsync(ipName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get Reserved IP operation retrieves the details for the virtual
        /// IP reserved for the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <param name='ipName'>
        /// Required. The name of the reserved IP to retrieve.
        /// </param>
        /// <returns>
        /// A reserved IP associated with your subscription.
        /// </returns>
        public static Task<NetworkReservedIPGetResponse> GetAsync(this IReservedIPOperations operations, string ipName)
        {
            return operations.GetAsync(ipName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List Reserved IP operation retrieves all of the virtual IPs
        /// reserved for the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <returns>
        /// The response structure for the Server List operation.
        /// </returns>
        public static NetworkReservedIPListResponse List(this IReservedIPOperations operations)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IReservedIPOperations)s).ListAsync();
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List Reserved IP operation retrieves all of the virtual IPs
        /// reserved for the subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Network.IReservedIPOperations.
        /// </param>
        /// <returns>
        /// The response structure for the Server List operation.
        /// </returns>
        public static Task<NetworkReservedIPListResponse> ListAsync(this IReservedIPOperations operations)
        {
            return operations.ListAsync(CancellationToken.None);
        }
    }
}
