// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure.Management.StorSimple.Models;

namespace Microsoft.WindowsAzure.Management.StorSimple
{
    /// <summary>
    /// All Operations related to virtual disk
    /// </summary>
    public partial interface IVirtualDiskOperations
    {
        /// <summary>
        /// The Begin Creating Volume operation creates a new volume.
        /// </summary>
        /// <param name='deviceId'>
        /// device id
        /// </param>
        /// <param name='diskDetails'>
        /// Parameters supplied to the Create virtual disk operation.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// The Custom Request Headers which client must use.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// This is the Task Response for all Async Calls
        /// </returns>
        Task<GuidTaskResponse> BeginCreatingAsync(string deviceId, VirtualDiskRequest diskDetails, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// The Begin Deleteing Volume operation deletes the specified volume.
        /// </summary>
        /// <param name='deviceId'>
        /// device id
        /// </param>
        /// <param name='diskId'>
        /// Instance id of the virtual disk to be deleted.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// The Custom Request Headers which client must use.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// This is the Task Response for all Async Calls
        /// </returns>
        Task<GuidTaskResponse> BeginDeletingAsync(string deviceId, string diskId, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// The Begin updating Volume operation updates an existing volume.
        /// </summary>
        /// <param name='deviceId'>
        /// device id
        /// </param>
        /// <param name='diskId'>
        /// disk id
        /// </param>
        /// <param name='diskDetails'>
        /// Parameters supplied to the update virtual disk operation.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// The Custom Request Headers which client must use.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// This is the Task Response for all Async Calls
        /// </returns>
        Task<GuidTaskResponse> BeginUpdatingAsync(string deviceId, string diskId, VirtualDisk diskDetails, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <param name='deviceId'>
        /// device id
        /// </param>
        /// <param name='diskDetails'>
        /// Parameters supplied to the Create virtual disk operation.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// The Custom Request Headers which client must use.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Info about the async task
        /// </returns>
        Task<TaskStatusInfo> CreateAsync(string deviceId, VirtualDiskRequest diskDetails, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <param name='deviceId'>
        /// device id
        /// </param>
        /// <param name='diskId'>
        /// disk id
        /// </param>
        /// <param name='customRequestHeaders'>
        /// The Custom Request Headers which client must use.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Info about the async task
        /// </returns>
        Task<TaskStatusInfo> DeleteAsync(string deviceId, string diskId, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the get of virtual disk.
        /// </returns>
        Task<VirtualDiskGetResponse> GetByNameAsync(string deviceId, string diskName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the list of virtual disks for a given data
        /// container.
        /// </returns>
        Task<VirtualDiskListResponse> ListAsync(string deviceId, string dataContainerId, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <param name='deviceId'>
        /// device id
        /// </param>
        /// <param name='diskId'>
        /// disk id
        /// </param>
        /// <param name='diskDetails'>
        /// Parameters supplied to the update virtual disk operation.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// The Custom Request Headers which client must use.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Info about the async task
        /// </returns>
        Task<TaskStatusInfo> UpdateAsync(string deviceId, string diskId, VirtualDisk diskDetails, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
    }
}
